#include "interruptcontrol.h"
#include <iostream>
#include <stdlib.h>
#include <Adafruit_Sensor.h>
#include <Wire.h>
#include <Arduino.h>

int shocktag = 0;
int angulartag = 0;


byte buff[6];

void ADXL375_Config(void){

  Wire.begin(SDA_ADXL, SCL_ADXL);
  pinMode(SDA_BNO055, INPUT);
  pinMode(SCL_BNO055, INPUT);
  
  /*START Disable Trigger Mode/enable Bypass Mode*/
  Wire.beginTransmission(ADXL375_Address);
  Wire.write(0x38); //FIFO_CTL Register Address
  Wire.write(0x2A); //Disable Trigger Mode, set samples = 10
  Wire.endTransmission();
  /*END Disable Trigger Mode/enable Bypass Mode*/

  /*START Enable Trigger Mode*/
  Wire.beginTransmission(ADXL375_Address);
  Wire.write(0x38); //FIFO_CTL Register Address
  Wire.write(0xEA); //Enable Trigger Mode, set samples = 10
  Wire.endTransmission();
  /*END Enable Trigger Mode*/

  //set shock threshold
  Wire.beginTransmission(ADXL375_Address);
  Wire.write(0x1D);
  Wire.write(0x26);  //30 g (.78 g per LSB)
  Wire.endTransmission();

  //set shock duration
  Wire.beginTransmission(ADXL375_Address);
  Wire.write(0x21);
  Wire.write(0x05); //10
  Wire.endTransmission();

  //set shock latency
  Wire.beginTransmission(ADXL375_Address);
  Wire.write(0x22);
  Wire.write(0x00); //disable second shock
  Wire.endTransmission();

  //set shock window
  Wire.beginTransmission(ADXL375_Address);
  Wire.write(0x23);
  Wire.write(0xF0); //300 ms
  Wire.endTransmission();

  //include all axes in shock detection
  Wire.beginTransmission(ADXL375_Address);
  Wire.write(0x2A);
  Wire.write(0x07);
  Wire.endTransmission();

  //set data rate
  Wire.beginTransmission(ADXL375_Address);
  Wire.write(0x2C);
  Wire.write(0x0F); //3200 Hz
  Wire.endTransmission();

  //endable single shock interrupt
  Wire.beginTransmission(ADXL375_Address);
  Wire.write(0x2E);
  Wire.write(0x40); //single shock
  Wire.endTransmission();

  //set single shock to int1 pin
  Wire.beginTransmission(ADXL375_Address);
  Wire.write(0x2F);
  Wire.write(0x00); //int 2 pin
  Wire.endTransmission();

  /*START Data Format*/
  Wire.beginTransmission(ADXL375_Address);
  Wire.write(0x31); //DATA_FORMAT Reg
  Wire.write(0x0B); 
  Wire.endTransmission();
  

  //start the measurement
  Wire.beginTransmission(ADXL375_Address);
  Wire.write(0x2D);
  Wire.write(0x08); //measurement mode
  Wire.endTransmission();

  Wire.end();

  attachInterrupt(6, ISR_SHOCKDETECT, RISING);
}

void ISR_SHOCKDETECT(){

detachInterrupt(7);

shocktag = 1;



}

void ReadFIFO(vector<double>& HIGH_G_DATA){

Wire.begin(SDA_ADXL, SCL_ADXL);
pinMode(SCL_BNO055, INPUT);
pinMode(SDA_BNO055, INPUT);

int16_t data_x = 0, data_x_lsb = 0; 
int16_t data_y = 0, data_y_lsb = 0; 
int16_t data_z = 0, data_z_lsb = 0;

Wire.beginTransmission(ADXL375_Address);
Wire.write(0x32); //read LSB
Wire.endTransmission();

Wire.requestFrom(ADXL375_Address, 6);   
while (Wire.available()) {
   data_x_lsb = Wire.read();
   data_x = Wire.read();
   data_y_lsb = Wire.read();
   data_y = Wire.read();
   data_z_lsb = Wire.read();
   data_z = Wire.read();

    data_x = (data_x << 8) | (data_x_lsb);
    data_y = (data_y << 8) | (data_y_lsb);
    data_z = (data_z << 8) | (data_z_lsb);
}

double data_x_1 = (double)data_x*.049;
double data_y_1 = (double)data_y*.049;
double data_z_1 = (double)data_z *.049;

HIGH_G_DATA.push_back(data_x_1);
HIGH_G_DATA.push_back(data_y_1);
HIGH_G_DATA.push_back(data_z_1);


//Serial.print("X: "); Serial.print(data_x_1); Serial.print("  Y: "); Serial.print(data_y_1);
//Serial.print("  Z: "); Serial.print(data_z_1); Serial.print("  "); Serial.println("g");

Wire.end();

return;

}

void ReadGyro(vector<double>& GYRO_DATA){

Wire.begin(SDA_BNO055, SCL_BNO055);
pinMode(SDA_ADXL, INPUT);
pinMode(SCL_ADXL, INPUT);

int16_t data_x = 0, data_x_lsb = 0; 
int16_t data_y = 0, data_y_lsb = 0; 
int16_t data_z = 0, data_z_lsb = 0;

//Register Page 0
Wire.beginTransmission(BNO055_Address);
Wire.write(0x07); //Register Page Sel 
Wire.write(0x00); //Register page 0;
Wire.endTransmission();

Wire.beginTransmission(BNO055_Address);
Wire.write(0x14);
Wire.endTransmission();

Wire.requestFrom(BNO055_Address, 6);   
while (Wire.available()) {
   data_x_lsb = Wire.read();
   data_x = Wire.read();
   data_y_lsb = Wire.read();
   data_y = Wire.read();
   data_z_lsb = Wire.read();
   data_z = Wire.read();

    data_x = (data_x << 8) | (data_x_lsb);
    data_y = (data_y << 8) | (data_y_lsb);
    data_z = (data_z << 8) | (data_z_lsb);
}

double data_x_1 = (double)data_x/900;
double data_y_1 = (double)data_y/900;
double data_z_1 = (double)data_z/900;

GYRO_DATA.push_back(data_x_1);
GYRO_DATA.push_back(data_y_1);
GYRO_DATA.push_back(data_z_1);

//Serial.print("X: "); Serial.print(data_x_1); Serial.print("  Y: "); Serial.print(data_y_1);
//Serial.print("  Z: "); Serial.print(data_z_1); Serial.print("  "); Serial.println("rad/s");





Wire.end();

}




void Clear_ADXL375_Interrupts(){

Wire.begin(SDA_ADXL, SCL_ADXL);
pinMode(SCL_BNO055, INPUT);
pinMode(SDA_BNO055, INPUT);


Wire.beginTransmission(ADXL375_Address);
Wire.write(0x30); //read INT_SOURCE REG to clear
Wire.endTransmission();

Wire.requestFrom(ADXL375_Address, 1);   
while (Wire.available()) {

  int result = Wire.read();


}

Wire.end();

return;

}

void Reset_ADXL375_Interrupts(){

  Wire.begin(SDA_ADXL, SCL_ADXL);
  pinMode(SDA_BNO055, INPUT);
  pinMode(SCL_BNO055, INPUT);

  /*START Disable Trigger Mode/enable Bypass Mode*/
  Wire.beginTransmission(ADXL375_Address);
  Wire.write(0x38); //FIFO_CTL Register Address
  Wire.write(0x2A); //Disable Trigger Mode, set samples = 10
  Wire.endTransmission();
  /*END Disable Trigger Mode/enable Bypass Mode*/

  /*START Enable Trigger Mode*/
  Wire.beginTransmission(ADXL375_Address);
  Wire.write(0x38); //FIFO_CTL Register Address
  Wire.write(0xEA); //Enable Trigger Mode, set samples = 10
  Wire.endTransmission();
  /*END Enable Trigger Mode*/

  Wire.end();

  return;

}

void BNO055_Config(){

Wire.begin(SDA_BNO055, SCL_BNO055);
pinMode(SDA_ADXL, INPUT);
pinMode(SCL_ADXL, INPUT);


//Register Page 0 
Wire.beginTransmission(BNO055_Address);
Wire.write(0x3D); //OP Mode reg
Wire.write(0x00); //config mode
Wire.endTransmission();

Wire.beginTransmission(BNO055_Address);
Wire.write(0x3F); //SYS_TRIGGER Register
Wire.write(0x40); //internal clock, reset int output
Wire.endTransmission();

Wire.beginTransmission(BNO055_Address);
Wire.write(0x3B); //Unit_SEL Register
Wire.write(0x02); //Rad/s for gyroscope data regs
Wire.endTransmission();

Wire.beginTransmission(BNO055_Address);
Wire.write(0x07); //Page SEL Register
Wire.write(0x01); //Change to register page 1
Wire.endTransmission();


//Register Page 1

Wire.beginTransmission(BNO055_Address);
Wire.write(0x0F); //INT_MASK Register
Wire.write(0x04); //map AM gyro int to pin
Wire.endTransmission();

Wire.beginTransmission(BNO055_Address);
Wire.write(0x10); //INT_EN Register
Wire.write(0x04); //Enable gyroscope any motion interrupt
Wire.endTransmission();

Wire.beginTransmission(BNO055_Address);
Wire.write(0x17); //GYR_INT_SETTING Register
Wire.write(0x47); //Unfiltered, x-y-z enabled data
Wire.endTransmission();

Wire.beginTransmission(BNO055_Address);
Wire.write(0x1E); //Gyro any-motion threshold calculator
Wire.write(0xF0); //36 deg/s for testing (save 0x24 = 36 deg/s)
Wire.endTransmission();

Wire.beginTransmission(BNO055_Address);
Wire.write(0x0A); //GYR_Config Reg
Wire.write(0x38); //2000 deg/s 400 hz for interrupt (00) set at 40 for testing
Wire.endTransmission();

Wire.beginTransmission(BNO055_Address);
Wire.write(0x1F); //GYR_AM_SET Reg
Wire.write(0x0B); //8 samples awake (int), 8 samples above the threshold to trigger
Wire.endTransmission();

//Register Page 0
Wire.beginTransmission(BNO055_Address);
Wire.write(0x07); //Register Page Sel 
Wire.write(0x00); //Register page 0;
Wire.endTransmission();

Wire.beginTransmission(BNO055_Address);
Wire.write(0x3D); //configuration register
Wire.write(0x03); //set to gyro mode
Wire.endTransmission();

attachInterrupt(7, ISR_ANGULARDETECT, RISING);

Wire.end();


return;

}

void Reset_BNO055_Interrupts(){

Wire.begin(SDA_BNO055, SCL_BNO055);
pinMode(SCL_ADXL, INPUT);
pinMode(SDA_ADXL, INPUT);

Wire.beginTransmission(BNO055_Address);
Wire.write(0x07); //Register Page Sel 
Wire.write(0x00); //Register page 0;
Wire.endTransmission();

Wire.beginTransmission(BNO055_Address);
Wire.write(0x3D); //OP Mode reg
Wire.write(0x00); //config mode
Wire.endTransmission();

Wire.beginTransmission(BNO055_Address);
Wire.write(0x3F); //SYS_TRIGGER Register
Wire.write(0x40); //internal clock, reset int output
Wire.endTransmission();

Wire.beginTransmission(BNO055_Address);
Wire.write(0x3D); //configuration register
Wire.write(0x03); //set to gyro mode
Wire.endTransmission();

Wire.end();

}

int Clear_BNO055_Interrupts(){

  Wire.begin(SDA_BNO055, SCL_BNO055);
  pinMode(SDA_ADXL, INPUT);
  pinMode(SCL_ADXL, INPUT);

  int intstatus;

  Wire.beginTransmission(BNO055_Address);
  Wire.write(0x07); //Page SEL Register
  Wire.write(0x01); //Change to register page 1
  Wire.endTransmission();


  Wire.beginTransmission(BNO055_Address);
  Wire.write(0x37); //int STA register
  Wire.endTransmission();

  Wire.requestFrom(BNO055_Address,1);
  while(Wire.available()){
    
    intstatus = Wire.read();
    

  }

  Wire.end();

  return intstatus;

}


void ISR_ANGULARDETECT(){

  detachInterrupt(6);
  detachInterrupt(7);

  angulartag = 1;

  return;

}





