#include <painlessMesh.h>
#include <Adafruit_ADXL375.h>
#include <Adafruit_Sensor.h>
#include <interruptcontrol.h>
#include <Adafruit_BNO055.h>
#include <sstream>
#include <string>
#include <esp_wifi.h>
#include <esp_now.h>


#define SIZE_DATA 15

#define NUMBER 2
#define SAMPLE_TIME 1/(32*4)

vector<double> HIGH_G_DATA;
vector<double> GYRO_DATA;
bool sendtag;

Adafruit_ADXL375 accel = Adafruit_ADXL375(12345);
Adafruit_BNO055 bno = Adafruit_BNO055(-1, 0x28, &Wire1);
Scheduler taskScheduler;
constexpr char WIFI_SSID[] = "SDNet";

uint8_t broadcastAddress[] = {0x58, 0xCF, 0x79, 0x16, 0x77, 0x7C}; // WiFi Hub Mac Address

typedef struct struct_message
{
  int number;
  double force[SIZE_DATA];
  double gyro[SIZE_DATA];
} struct_message;

struct_message myData;
esp_now_peer_info_t peerInfo;

void detectConcussion();
void sendMessage();

Task taskDetectConcussion(0, TASK_FOREVER, &detectConcussion);
Task taskSendAlert(0, TASK_FOREVER, &sendMessage);

bool concussionDetected = false;
bool angular = false;
bool impact = false;

void sendMessage()
{
  Serial.printf("Trying to send message");
  if (impact == true)
  {
    // Dont do anything here
    myData.number = NUMBER;
    int count=0;
    for (int i = 14; i < 30; i++)
    {
        if (HIGH_G_DATA[i] <= -120 || HIGH_G_DATA[i] >= 120)
        {
          myData.gyro[count] = 0;
          myData.force[count] = 0;
        }
        else
        {
          myData.gyro[count] = 0;
          myData.force[count] = HIGH_G_DATA[i];
        }
        count++;
    }
  }
  if (angular == true)
  {
    // x,y,z format each time for rotational -- these are velocity in rad/s ... need in acceleration rad/s^2. so we need to find slope of the values
    // time being 1/400 seconds = variable
    // finding the slope between each one
    // range 1500 ~+=1000
    myData.number = NUMBER;
    for (int i = 0; i < SIZE_DATA; i++)
    {
          myData.gyro[i] = GYRO_DATA[i];
          myData.force[i] = 0;
    }
  }

    for (int i = 0; i < SIZE_DATA; i++)
    {
        Serial.println(myData.force[i]);
    }
    for (int i = 0; i < SIZE_DATA; i++)
    {
        Serial.println(myData.gyro[i]);
    }

    esp_err_t result = esp_now_send(broadcastAddress, (uint8_t *)&myData, sizeof(myData));
    Serial.println("just tried to send");
    if (result == ESP_OK)
    {
        Serial.println("Sent with success");
    }
    else
    {
        Serial.println("Error sending the data");
    }

    concussionDetected = false;
    angular = false;
    impact = false;
    HIGH_G_DATA.clear();
    GYRO_DATA.clear();
    taskDetectConcussion.enable(); // Re-enable concussion detectiaton task
    taskSendAlert.disable();
    attachInterrupt(7, ISR_ANGULARDETECT, RISING);
    Reset_BNO055_Interrupts();
  }

void OnDataSent(const uint8_t *mac_addr, esp_now_send_status_t status)
{
  Serial.print("\r\nLast Packet Send Status:\t");
  Serial.println(status == ESP_NOW_SEND_SUCCESS ? "Delivery Success" : "Delivery Fail");
}

int32_t getWiFiChannel(const char *ssid)
{
  if (int32_t n = WiFi.scanNetworks())
  {
    for (uint8_t i = 0; i < n; i++)
    {
      if (!strcmp(ssid, WiFi.SSID(i).c_str()))
      {
        return WiFi.channel(i);
      }
    }
  }
  return 0;
}

void setup()
{
  Serial.begin(115200);

  WiFi.mode(WIFI_STA);

  int32_t channel = getWiFiChannel(WIFI_SSID);
  //Serial.println("Channel: ");
  //Serial.println(channel);

  // WiFi.printDiag(Serial); // Uncomment to verify channel number before
  esp_wifi_set_promiscuous(true);
  esp_wifi_set_channel(channel, WIFI_SECOND_CHAN_NONE);
  esp_wifi_set_promiscuous(false);

  if (esp_now_init() != ESP_OK)
  {
    Serial.println("Error initializing ESP-NOW");
    return;
  }

  // Register peer
  memcpy(peerInfo.peer_addr, broadcastAddress, 6);
  peerInfo.channel = 0;
  peerInfo.encrypt = false;

  if (esp_now_add_peer(&peerInfo) != ESP_OK)
  {
    Serial.println("Failed to add peer");
    return;
  }

  esp_now_register_send_cb(OnDataSent);

  taskScheduler.addTask(taskDetectConcussion);
  taskScheduler.addTask(taskSendAlert);
  taskDetectConcussion.enable();

  // Sensor setup
  pinMode(6, INPUT);
  pinMode(7, INPUT);

  Clear_ADXL375_Interrupts();
  Reset_ADXL375_Interrupts();
  ADXL375_Config();

  Clear_BNO055_Interrupts();
  Reset_BNO055_Interrupts();
  BNO055_Config();

}

void detectConcussion()
{
  // if data > threshold then
  // greg data here <3

  HIGH_G_DATA.clear();
  GYRO_DATA.clear();

  if (shocktag)
  {

    shocktag = 0;

    for (int i = 0; i < 30; i++)
    {
      ReadFIFO(HIGH_G_DATA);
    }

    for (int i = 0; i < 30; i++)
    {
      if (HIGH_G_DATA.at(i) > 30)
      {
        sendtag = true;
      }
    }

    if (sendtag)
    {
      /*for (int j = 0; j < 15; j++)
      {
         Serial.println(HIGH_G_DATA.at(j));
      }*/

      concussionDetected = true;
      impact = true;
      taskDetectConcussion.disable(); // Disable concussion detection task
      taskSendAlert.enable();         // Enable alert task
      sendtag = false;
    }

    Clear_ADXL375_Interrupts();
    Reset_ADXL375_Interrupts();
    attachInterrupt(7, ISR_ANGULARDETECT, RISING);
  }

  if (angulartag)
  {

    // Serial.print("INT TRIGGERED");

    angulartag = 0;

    for (int i = 0; i < 15; i++)
    {
      ReadGyro(GYRO_DATA);
    }

    /*for (int j = 0; j < 15; j++)
    {
       Serial.println(GYRO_DATA.at(j));
    }*/

    concussionDetected = true;
    angular = true;
    taskDetectConcussion.disable(); // Disable concussion detection task
    taskSendAlert.enable();         // Enable alert task

    attachInterrupt(6, ISR_SHOCKDETECT, RISING);
  }
}

void loop()
{
  taskScheduler.execute();
}